from os.path import dirname, join
import pycrfsuite
from transformer import Transformer
from underthesea.util.singleton import Singleton


@Singleton
class ChunkingCRFModel:
    def __init__(self):
        self.model = pycrfsuite.Tagger()
        filepath = join(dirname(__file__), "chunking_crf_v1.model")
        self.model.open(filepath)

    def predict(self, sentence, format=None):
        x = Transformer.transform(sentence)
        tags = self.model.tag(x)
        tokenized_sentence = u''
        output = [(token[0], token[1], tag) for token, tag in zip(sentence, tags)]
        return output



from os.path import dirname
from os.path import join

from underthesea.pos_tag.feature import word2features


def sent2features(sent, template):
    return [word2features(sent, i, template) for i in range(len(sent))]


class Transformer:
    def __init__(self):
        pass

    @staticmethod
    def transform(sentence):
        template = [
            "T[0].lower", "T[-1].lower", "T[1].lower",
            "T[0].istitle", "T[-1].istitle", "T[1].istitle",
            "T[-2]", "T[-1]", "T[0]", "T[1]", "T[2]",  # word unigram
            "T[-2,-1]", "T[-1,0]", "T[0,1]", "T[1,2]",  # bigram
            "T[-1][1]", "T[-2][1]", "T[-3][1]",  # pos
            "T[-3,-2][1]", "T[-2,-1][1]",  # pos bigram
            "T[-3,-1][1]"  # pos trigram
        ]
        sentence = [(token[0], token[1], "A") for token in sentence]
        return sent2features(sentence, template)

    @staticmethod
    def extract_features(sentence, template):
        return sent2features(sentence, template)


def sent2labels(sent):
    return [label for token, label in sent]



from underthesea import pos_tag
from underthesea.chunking.model import ChunkingCRFModel


def chunk(sentence, format=None):
    """
    chunk a sentence to phrases 
    
    :param unicode sentence: raw sentence
    :return: list of tuple with word, pos tag, chunking tag 
    :rtype: list 
    """
    sentence = pos_tag(sentence)
    crf_model = ChunkingCRFModel.Instance()
    result = crf_model.predict(sentence, format)
    return result

